@@ -161,7 +161,6 @@ bitflags! {
 mod x86_64 {
     use super::*;
     use crate::{instructions::tlb::Pcid, structures::paging::PhysFrame, PhysAddr, VirtAddr};
-    #[cfg(feature = "inline_asm")]
     use core::arch::asm;
 
     impl Cr0 {
@@ -176,14 +175,9 @@ mod x86_64 {
         pub fn read_raw() -> u64 {
             let value: u64;
 
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov {}, cr0", out(reg) value, options(nomem, nostack, preserves_flags));
             }
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                value = crate::asm::x86_64_asm_read_cr0();
-            }
 
             value
         }
@@ -217,15 +211,9 @@ mod x86_64 {
         /// safety through it, e.g. by disabling paging.
         #[inline]
         pub unsafe fn write_raw(value: u64) {
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov cr0, {}", in(reg) value, options(nostack, preserves_flags));
             }
-
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                crate::asm::x86_64_asm_write_cr0(value);
-            }
         }
 
         /// Updates CR0 flags.
@@ -261,14 +249,9 @@ mod x86_64 {
         pub fn read_raw() -> u64 {
             let value: u64;
 
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov {}, cr2", out(reg) value, options(nomem, nostack, preserves_flags));
             }
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                value = crate::asm::x86_64_asm_read_cr2();
-            }
 
             value
         }
@@ -288,14 +271,9 @@ mod x86_64 {
         pub fn read_raw() -> (PhysFrame, u16) {
             let value: u64;
 
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov {}, cr3", out(reg) value, options(nomem, nostack, preserves_flags));
             }
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                value = crate::asm::x86_64_asm_read_cr3();
-            }
 
             let addr = PhysAddr::new(value & 0x_000f_ffff_ffff_f000);
             let frame = PhysFrame::containing_address(addr);
@@ -349,15 +327,9 @@ mod x86_64 {
             let addr = frame.start_address();
             let value = addr.as_u64() | val as u64;
 
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov cr3, {}", in(reg) value, options(nostack, preserves_flags));
             }
-
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                crate::asm::x86_64_asm_write_cr3(value)
-            }
         }
     }
 
@@ -373,14 +345,9 @@ mod x86_64 {
         pub fn read_raw() -> u64 {
             let value: u64;
 
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov {}, cr4", out(reg) value, options(nomem, nostack, preserves_flags));
             }
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                value = crate::asm::x86_64_asm_read_cr4();
-            }
 
             value
         }
@@ -416,15 +383,9 @@ mod x86_64 {
         /// flag.
         #[inline]
         pub unsafe fn write_raw(value: u64) {
-            #[cfg(feature = "inline_asm")]
             unsafe {
                 asm!("mov cr4, {}", in(reg) value, options(nostack, preserves_flags));
             }
-
-            #[cfg(not(feature = "inline_asm"))]
-            unsafe {
-                crate::asm::x86_64_asm_write_cr4(value);
-            }
         }
 
         /// Updates CR4 flags.
