@@ -1,13 +1,14 @@
 //! Provides a type for the task state segment structure.
 
 use crate::VirtAddr;
+use core::mem::size_of;
 
 /// In 64-bit mode the TSS holds information that is not
 /// directly related to the task-switch mechanism,
 /// but is used for finding kernel level stack
 /// if interrupts arrive while in kernel mode.
 #[derive(Debug, Clone, Copy)]
-#[repr(C, packed)]
+#[repr(C, packed(4))]
 pub struct TaskStateSegment {
     reserved_1: u32,
     /// The full 64-bit canonical forms of the stack pointers (RSP) for privilege levels 0-2.
@@ -22,14 +23,18 @@ pub struct TaskStateSegment {
 }
 
 impl TaskStateSegment {
-    /// Creates a new TSS with zeroed privilege and interrupt stack table and a zero
-    /// `iomap_base`.
+    /// Creates a new TSS with zeroed privilege and interrupt stack table and an
+    /// empty I/O-Permission Bitmap.
+    ///
+    /// As we always set the TSS segment limit to
+    /// `size_of::<TaskStateSegment>() - 1`, this means that `iomap_base` is
+    /// initialized to `size_of::<TaskStateSegment>()`.
     #[inline]
     pub const fn new() -> TaskStateSegment {
         TaskStateSegment {
             privilege_stack_table: [VirtAddr::zero(); 3],
             interrupt_stack_table: [VirtAddr::zero(); 7],
-            iomap_base: 0,
+            iomap_base: size_of::<TaskStateSegment>() as u16,
             reserved_1: 0,
             reserved_2: 0,
             reserved_3: 0,
@@ -37,3 +42,15 @@ impl TaskStateSegment {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    pub fn check_tss_size() {
+        // Per the SDM, the minimum size of a TSS is 0x68 bytes, giving a
+        // minimum limit of 0x67.
+        assert_eq!(size_of::<TaskStateSegment>(), 0x68);
+    }
+}
